#include "rain.h"
#include "constants.h"
#include "overlay.h"
#include <vector>

#define COLOR_HIGHLIGHT_R 128
#define COLOR_HIGHLIGHT_G 255
#define COLOR_HIGHLIGHT_B 128

#define COLOR_DEFAULT_R 0
#define COLOR_DEFAULT_G 255
#define COLOR_DEFAULT_B 0

//typedef struct {
struct Raindrop {
    //bool reached_bottom = false;
    bool reached_bottom;
    int x;
    int y;
    int length;
    int speed;
    unsigned long last_moved;
} ;

bool rain_initialized = false;
uint16_t color_highlight;
uint16_t color_default;
std::vector<Raindrop> raindrops;

void setup_rain(MatrixPanel_I2S_DMA *matrix) {
  color_highlight = matrix->color565(
      COLOR_HIGHLIGHT_R,
      COLOR_HIGHLIGHT_G,
      COLOR_HIGHLIGHT_B
  );
  color_default = matrix->color565(
      COLOR_DEFAULT_R,
      COLOR_DEFAULT_B,
      COLOR_DEFAULT_G
  );
  for (int x = 0; x<PANEL_WIDTH; x++) {
    raindrops.push_back({
          false,
          x,
          0 - random(24),
          random(20, 28),
          random(25, 100),
          millis()
    });
  }
}

void draw_rain(MatrixPanel_I2S_DMA *matrix, Mode mode) {
  //matrix ->drawBitmap(0, 0, overlay, 64, 32, matrix->color565(64, 0, 0));
  if (!rain_initialized) {
    setup_rain(matrix);
    rain_initialized = true;
  }
  unsigned long timestamp = millis();
  int x = 0;
  while(x < raindrops.size()) {
    if ((timestamp - raindrops[x].last_moved) > raindrops[x].speed) {
      // step down a pixel
      raindrops[x].y++;
      raindrops[x].last_moved = timestamp;
    }
    if (raindrops[x].y > PANEL_HEIGHT + raindrops[x].length) {
      raindrops.push_back({
          false,
          raindrops[x].x,
          0 - random(8),
          random(20, 28),
          random(0, 76),
          millis()
      });
      raindrops.erase(raindrops.begin() + x);
      continue;
    } 
    if (raindrops[x].y >= 0) {
      //draw our pixel
      matrix->drawPixel(raindrops[x].x, raindrops[x].y, matrix->color565(128, 255, 128));
      // draw our trail
      for (int trail_offset = 1; trail_offset < raindrops[x].length; trail_offset++) {
        int trail_y = raindrops[x].y - trail_offset;
        float brightness = ((raindrops[x].length - trail_offset) / (float)raindrops[x].length);
        uint16_t fade_color = matrix->color565(
            COLOR_DEFAULT_R * brightness,
            COLOR_DEFAULT_G * brightness,
            COLOR_DEFAULT_B * brightness
        );

        if (trail_y < 0) {
          break;
        }

        int pixel_num = (trail_y * PANEL_WIDTH) + raindrops[x].x;
        if (overlay[pixel_num / 8] & (1 << (7 - (pixel_num % 8))) and mode != Stealth) {
          if (mode == HighVis) {
            matrix->drawPixel(raindrops[x].x, trail_y, color_highlight);
          } else {
            matrix->drawPixel(raindrops[x].x, trail_y, matrix->color565(
                    COLOR_HIGHLIGHT_R * brightness,
                    COLOR_HIGHLIGHT_G * brightness,
                    COLOR_HIGHLIGHT_B * brightness
            ));
          }
        } else {
          matrix->drawPixel(raindrops[x].x, trail_y, fade_color);
        }
      }
    }
    x++;
  }
}
